import { RequestHandler, ResponseResolver, MockedRequest, DefaultRequestBodyType, RequestParams } from './utils/handlers/requestHandler';
import { Mask } from './setupWorker/glossary';
import { set } from './context/set';
import { matchRequestUrl } from './utils/matching/matchRequestUrl';
export declare enum RESTMethods {
    HEAD = "HEAD",
    GET = "GET",
    POST = "POST",
    PUT = "PUT",
    PATCH = "PATCH",
    OPTIONS = "OPTIONS",
    DELETE = "DELETE"
}
export declare const restContext: {
    set: typeof set;
    status: (statusCode: number, statusText?: string | undefined) => import("./response").ResponseTransformer<any>;
    cookie: (name: string, value: string, options?: import("cookie").CookieSerializeOptions | undefined) => import("./response").ResponseTransformer<any>;
    body: <BodyType extends string | Blob | ArrayBufferView | ArrayBuffer | ReadableStream<any> | FormData>(value: BodyType) => import("./response").ResponseTransformer<BodyType>;
    text: <BodyType_1 extends string>(body: BodyType_1) => import("./response").ResponseTransformer<BodyType_1>;
    json: <BodyTypeJSON>(body: BodyTypeJSON, { merge }?: {
        merge?: boolean | undefined;
    }) => import("./response").ResponseTransformer<BodyTypeJSON>;
    xml: <BodyType_2 extends string>(body: BodyType_2) => import("./response").ResponseTransformer<BodyType_2>;
    delay: (durationMs?: number | undefined) => import("./response").ResponseTransformer<any>;
    fetch: (input: string | MockedRequest<DefaultRequestBodyType, RequestParams>, requestInit?: RequestInit) => any;
};
export interface ParsedRestRequest {
    match: ReturnType<typeof matchRequestUrl>;
}
export declare const rest: {
    head: <RequestBodyType = DefaultRequestBodyType, ResponseBodyType = any, RequestParamsType extends RequestParams = RequestParams>(mask: Mask, resolver: ResponseResolver<MockedRequest<RequestBodyType, RequestParamsType>, {
        set: typeof set;
        status: (statusCode: number, statusText?: string | undefined) => import("./response").ResponseTransformer<any>;
        cookie: (name: string, value: string, options?: import("cookie").CookieSerializeOptions | undefined) => import("./response").ResponseTransformer<any>;
        body: <BodyType extends string | Blob | ArrayBufferView | ArrayBuffer | ReadableStream<any> | FormData>(value: BodyType) => import("./response").ResponseTransformer<BodyType>;
        text: <BodyType_1 extends string>(body: BodyType_1) => import("./response").ResponseTransformer<BodyType_1>;
        json: <BodyTypeJSON>(body: BodyTypeJSON, { merge }?: {
            merge?: boolean | undefined;
        }) => import("./response").ResponseTransformer<BodyTypeJSON>;
        xml: <BodyType_2 extends string>(body: BodyType_2) => import("./response").ResponseTransformer<BodyType_2>;
        delay: (durationMs?: number | undefined) => import("./response").ResponseTransformer<any>;
        fetch: (input: string | MockedRequest<DefaultRequestBodyType, RequestParams>, requestInit?: RequestInit) => any;
    }, ResponseBodyType>) => RequestHandler<MockedRequest<RequestBodyType, RequestParamsType>, {
        set: typeof set;
        status: (statusCode: number, statusText?: string | undefined) => import("./response").ResponseTransformer<any>;
        cookie: (name: string, value: string, options?: import("cookie").CookieSerializeOptions | undefined) => import("./response").ResponseTransformer<any>;
        body: <BodyType extends string | Blob | ArrayBufferView | ArrayBuffer | ReadableStream<any> | FormData>(value: BodyType) => import("./response").ResponseTransformer<BodyType>;
        text: <BodyType_1 extends string>(body: BodyType_1) => import("./response").ResponseTransformer<BodyType_1>;
        json: <BodyTypeJSON>(body: BodyTypeJSON, { merge }?: {
            merge?: boolean | undefined;
        }) => import("./response").ResponseTransformer<BodyTypeJSON>;
        xml: <BodyType_2 extends string>(body: BodyType_2) => import("./response").ResponseTransformer<BodyType_2>;
        delay: (durationMs?: number | undefined) => import("./response").ResponseTransformer<any>;
        fetch: (input: string | MockedRequest<DefaultRequestBodyType, RequestParams>, requestInit?: RequestInit) => any;
    }, ParsedRestRequest, MockedRequest<RequestBodyType, RequestParams>, ResponseBodyType>;
    get: <RequestBodyType = DefaultRequestBodyType, ResponseBodyType = any, RequestParamsType extends RequestParams = RequestParams>(mask: Mask, resolver: ResponseResolver<MockedRequest<RequestBodyType, RequestParamsType>, {
        set: typeof set;
        status: (statusCode: number, statusText?: string | undefined) => import("./response").ResponseTransformer<any>;
        cookie: (name: string, value: string, options?: import("cookie").CookieSerializeOptions | undefined) => import("./response").ResponseTransformer<any>;
        body: <BodyType extends string | Blob | ArrayBufferView | ArrayBuffer | ReadableStream<any> | FormData>(value: BodyType) => import("./response").ResponseTransformer<BodyType>;
        text: <BodyType_1 extends string>(body: BodyType_1) => import("./response").ResponseTransformer<BodyType_1>;
        json: <BodyTypeJSON>(body: BodyTypeJSON, { merge }?: {
            merge?: boolean | undefined;
        }) => import("./response").ResponseTransformer<BodyTypeJSON>;
        xml: <BodyType_2 extends string>(body: BodyType_2) => import("./response").ResponseTransformer<BodyType_2>;
        delay: (durationMs?: number | undefined) => import("./response").ResponseTransformer<any>;
        fetch: (input: string | MockedRequest<DefaultRequestBodyType, RequestParams>, requestInit?: RequestInit) => any;
    }, ResponseBodyType>) => RequestHandler<MockedRequest<RequestBodyType, RequestParamsType>, {
        set: typeof set;
        status: (statusCode: number, statusText?: string | undefined) => import("./response").ResponseTransformer<any>;
        cookie: (name: string, value: string, options?: import("cookie").CookieSerializeOptions | undefined) => import("./response").ResponseTransformer<any>;
        body: <BodyType extends string | Blob | ArrayBufferView | ArrayBuffer | ReadableStream<any> | FormData>(value: BodyType) => import("./response").ResponseTransformer<BodyType>;
        text: <BodyType_1 extends string>(body: BodyType_1) => import("./response").ResponseTransformer<BodyType_1>;
        json: <BodyTypeJSON>(body: BodyTypeJSON, { merge }?: {
            merge?: boolean | undefined;
        }) => import("./response").ResponseTransformer<BodyTypeJSON>;
        xml: <BodyType_2 extends string>(body: BodyType_2) => import("./response").ResponseTransformer<BodyType_2>;
        delay: (durationMs?: number | undefined) => import("./response").ResponseTransformer<any>;
        fetch: (input: string | MockedRequest<DefaultRequestBodyType, RequestParams>, requestInit?: RequestInit) => any;
    }, ParsedRestRequest, MockedRequest<RequestBodyType, RequestParams>, ResponseBodyType>;
    post: <RequestBodyType = DefaultRequestBodyType, ResponseBodyType = any, RequestParamsType extends RequestParams = RequestParams>(mask: Mask, resolver: ResponseResolver<MockedRequest<RequestBodyType, RequestParamsType>, {
        set: typeof set;
        status: (statusCode: number, statusText?: string | undefined) => import("./response").ResponseTransformer<any>;
        cookie: (name: string, value: string, options?: import("cookie").CookieSerializeOptions | undefined) => import("./response").ResponseTransformer<any>;
        body: <BodyType extends string | Blob | ArrayBufferView | ArrayBuffer | ReadableStream<any> | FormData>(value: BodyType) => import("./response").ResponseTransformer<BodyType>;
        text: <BodyType_1 extends string>(body: BodyType_1) => import("./response").ResponseTransformer<BodyType_1>;
        json: <BodyTypeJSON>(body: BodyTypeJSON, { merge }?: {
            merge?: boolean | undefined;
        }) => import("./response").ResponseTransformer<BodyTypeJSON>;
        xml: <BodyType_2 extends string>(body: BodyType_2) => import("./response").ResponseTransformer<BodyType_2>;
        delay: (durationMs?: number | undefined) => import("./response").ResponseTransformer<any>;
        fetch: (input: string | MockedRequest<DefaultRequestBodyType, RequestParams>, requestInit?: RequestInit) => any;
    }, ResponseBodyType>) => RequestHandler<MockedRequest<RequestBodyType, RequestParamsType>, {
        set: typeof set;
        status: (statusCode: number, statusText?: string | undefined) => import("./response").ResponseTransformer<any>;
        cookie: (name: string, value: string, options?: import("cookie").CookieSerializeOptions | undefined) => import("./response").ResponseTransformer<any>;
        body: <BodyType extends string | Blob | ArrayBufferView | ArrayBuffer | ReadableStream<any> | FormData>(value: BodyType) => import("./response").ResponseTransformer<BodyType>;
        text: <BodyType_1 extends string>(body: BodyType_1) => import("./response").ResponseTransformer<BodyType_1>;
        json: <BodyTypeJSON>(body: BodyTypeJSON, { merge }?: {
            merge?: boolean | undefined;
        }) => import("./response").ResponseTransformer<BodyTypeJSON>;
        xml: <BodyType_2 extends string>(body: BodyType_2) => import("./response").ResponseTransformer<BodyType_2>;
        delay: (durationMs?: number | undefined) => import("./response").ResponseTransformer<any>;
        fetch: (input: string | MockedRequest<DefaultRequestBodyType, RequestParams>, requestInit?: RequestInit) => any;
    }, ParsedRestRequest, MockedRequest<RequestBodyType, RequestParams>, ResponseBodyType>;
    put: <RequestBodyType = DefaultRequestBodyType, ResponseBodyType = any, RequestParamsType extends RequestParams = RequestParams>(mask: Mask, resolver: ResponseResolver<MockedRequest<RequestBodyType, RequestParamsType>, {
        set: typeof set;
        status: (statusCode: number, statusText?: string | undefined) => import("./response").ResponseTransformer<any>;
        cookie: (name: string, value: string, options?: import("cookie").CookieSerializeOptions | undefined) => import("./response").ResponseTransformer<any>;
        body: <BodyType extends string | Blob | ArrayBufferView | ArrayBuffer | ReadableStream<any> | FormData>(value: BodyType) => import("./response").ResponseTransformer<BodyType>;
        text: <BodyType_1 extends string>(body: BodyType_1) => import("./response").ResponseTransformer<BodyType_1>;
        json: <BodyTypeJSON>(body: BodyTypeJSON, { merge }?: {
            merge?: boolean | undefined;
        }) => import("./response").ResponseTransformer<BodyTypeJSON>;
        xml: <BodyType_2 extends string>(body: BodyType_2) => import("./response").ResponseTransformer<BodyType_2>;
        delay: (durationMs?: number | undefined) => import("./response").ResponseTransformer<any>;
        fetch: (input: string | MockedRequest<DefaultRequestBodyType, RequestParams>, requestInit?: RequestInit) => any;
    }, ResponseBodyType>) => RequestHandler<MockedRequest<RequestBodyType, RequestParamsType>, {
        set: typeof set;
        status: (statusCode: number, statusText?: string | undefined) => import("./response").ResponseTransformer<any>;
        cookie: (name: string, value: string, options?: import("cookie").CookieSerializeOptions | undefined) => import("./response").ResponseTransformer<any>;
        body: <BodyType extends string | Blob | ArrayBufferView | ArrayBuffer | ReadableStream<any> | FormData>(value: BodyType) => import("./response").ResponseTransformer<BodyType>;
        text: <BodyType_1 extends string>(body: BodyType_1) => import("./response").ResponseTransformer<BodyType_1>;
        json: <BodyTypeJSON>(body: BodyTypeJSON, { merge }?: {
            merge?: boolean | undefined;
        }) => import("./response").ResponseTransformer<BodyTypeJSON>;
        xml: <BodyType_2 extends string>(body: BodyType_2) => import("./response").ResponseTransformer<BodyType_2>;
        delay: (durationMs?: number | undefined) => import("./response").ResponseTransformer<any>;
        fetch: (input: string | MockedRequest<DefaultRequestBodyType, RequestParams>, requestInit?: RequestInit) => any;
    }, ParsedRestRequest, MockedRequest<RequestBodyType, RequestParams>, ResponseBodyType>;
    delete: <RequestBodyType = DefaultRequestBodyType, ResponseBodyType = any, RequestParamsType extends RequestParams = RequestParams>(mask: Mask, resolver: ResponseResolver<MockedRequest<RequestBodyType, RequestParamsType>, {
        set: typeof set;
        status: (statusCode: number, statusText?: string | undefined) => import("./response").ResponseTransformer<any>;
        cookie: (name: string, value: string, options?: import("cookie").CookieSerializeOptions | undefined) => import("./response").ResponseTransformer<any>;
        body: <BodyType extends string | Blob | ArrayBufferView | ArrayBuffer | ReadableStream<any> | FormData>(value: BodyType) => import("./response").ResponseTransformer<BodyType>;
        text: <BodyType_1 extends string>(body: BodyType_1) => import("./response").ResponseTransformer<BodyType_1>;
        json: <BodyTypeJSON>(body: BodyTypeJSON, { merge }?: {
            merge?: boolean | undefined;
        }) => import("./response").ResponseTransformer<BodyTypeJSON>;
        xml: <BodyType_2 extends string>(body: BodyType_2) => import("./response").ResponseTransformer<BodyType_2>;
        delay: (durationMs?: number | undefined) => import("./response").ResponseTransformer<any>;
        fetch: (input: string | MockedRequest<DefaultRequestBodyType, RequestParams>, requestInit?: RequestInit) => any;
    }, ResponseBodyType>) => RequestHandler<MockedRequest<RequestBodyType, RequestParamsType>, {
        set: typeof set;
        status: (statusCode: number, statusText?: string | undefined) => import("./response").ResponseTransformer<any>;
        cookie: (name: string, value: string, options?: import("cookie").CookieSerializeOptions | undefined) => import("./response").ResponseTransformer<any>;
        body: <BodyType extends string | Blob | ArrayBufferView | ArrayBuffer | ReadableStream<any> | FormData>(value: BodyType) => import("./response").ResponseTransformer<BodyType>;
        text: <BodyType_1 extends string>(body: BodyType_1) => import("./response").ResponseTransformer<BodyType_1>;
        json: <BodyTypeJSON>(body: BodyTypeJSON, { merge }?: {
            merge?: boolean | undefined;
        }) => import("./response").ResponseTransformer<BodyTypeJSON>;
        xml: <BodyType_2 extends string>(body: BodyType_2) => import("./response").ResponseTransformer<BodyType_2>;
        delay: (durationMs?: number | undefined) => import("./response").ResponseTransformer<any>;
        fetch: (input: string | MockedRequest<DefaultRequestBodyType, RequestParams>, requestInit?: RequestInit) => any;
    }, ParsedRestRequest, MockedRequest<RequestBodyType, RequestParams>, ResponseBodyType>;
    patch: <RequestBodyType = DefaultRequestBodyType, ResponseBodyType = any, RequestParamsType extends RequestParams = RequestParams>(mask: Mask, resolver: ResponseResolver<MockedRequest<RequestBodyType, RequestParamsType>, {
        set: typeof set;
        status: (statusCode: number, statusText?: string | undefined) => import("./response").ResponseTransformer<any>;
        cookie: (name: string, value: string, options?: import("cookie").CookieSerializeOptions | undefined) => import("./response").ResponseTransformer<any>;
        body: <BodyType extends string | Blob | ArrayBufferView | ArrayBuffer | ReadableStream<any> | FormData>(value: BodyType) => import("./response").ResponseTransformer<BodyType>;
        text: <BodyType_1 extends string>(body: BodyType_1) => import("./response").ResponseTransformer<BodyType_1>;
        json: <BodyTypeJSON>(body: BodyTypeJSON, { merge }?: {
            merge?: boolean | undefined;
        }) => import("./response").ResponseTransformer<BodyTypeJSON>;
        xml: <BodyType_2 extends string>(body: BodyType_2) => import("./response").ResponseTransformer<BodyType_2>;
        delay: (durationMs?: number | undefined) => import("./response").ResponseTransformer<any>;
        fetch: (input: string | MockedRequest<DefaultRequestBodyType, RequestParams>, requestInit?: RequestInit) => any;
    }, ResponseBodyType>) => RequestHandler<MockedRequest<RequestBodyType, RequestParamsType>, {
        set: typeof set;
        status: (statusCode: number, statusText?: string | undefined) => import("./response").ResponseTransformer<any>;
        cookie: (name: string, value: string, options?: import("cookie").CookieSerializeOptions | undefined) => import("./response").ResponseTransformer<any>;
        body: <BodyType extends string | Blob | ArrayBufferView | ArrayBuffer | ReadableStream<any> | FormData>(value: BodyType) => import("./response").ResponseTransformer<BodyType>;
        text: <BodyType_1 extends string>(body: BodyType_1) => import("./response").ResponseTransformer<BodyType_1>;
        json: <BodyTypeJSON>(body: BodyTypeJSON, { merge }?: {
            merge?: boolean | undefined;
        }) => import("./response").ResponseTransformer<BodyTypeJSON>;
        xml: <BodyType_2 extends string>(body: BodyType_2) => import("./response").ResponseTransformer<BodyType_2>;
        delay: (durationMs?: number | undefined) => import("./response").ResponseTransformer<any>;
        fetch: (input: string | MockedRequest<DefaultRequestBodyType, RequestParams>, requestInit?: RequestInit) => any;
    }, ParsedRestRequest, MockedRequest<RequestBodyType, RequestParams>, ResponseBodyType>;
    options: <RequestBodyType = DefaultRequestBodyType, ResponseBodyType = any, RequestParamsType extends RequestParams = RequestParams>(mask: Mask, resolver: ResponseResolver<MockedRequest<RequestBodyType, RequestParamsType>, {
        set: typeof set;
        status: (statusCode: number, statusText?: string | undefined) => import("./response").ResponseTransformer<any>;
        cookie: (name: string, value: string, options?: import("cookie").CookieSerializeOptions | undefined) => import("./response").ResponseTransformer<any>;
        body: <BodyType extends string | Blob | ArrayBufferView | ArrayBuffer | ReadableStream<any> | FormData>(value: BodyType) => import("./response").ResponseTransformer<BodyType>;
        text: <BodyType_1 extends string>(body: BodyType_1) => import("./response").ResponseTransformer<BodyType_1>;
        json: <BodyTypeJSON>(body: BodyTypeJSON, { merge }?: {
            merge?: boolean | undefined;
        }) => import("./response").ResponseTransformer<BodyTypeJSON>;
        xml: <BodyType_2 extends string>(body: BodyType_2) => import("./response").ResponseTransformer<BodyType_2>;
        delay: (durationMs?: number | undefined) => import("./response").ResponseTransformer<any>;
        fetch: (input: string | MockedRequest<DefaultRequestBodyType, RequestParams>, requestInit?: RequestInit) => any;
    }, ResponseBodyType>) => RequestHandler<MockedRequest<RequestBodyType, RequestParamsType>, {
        set: typeof set;
        status: (statusCode: number, statusText?: string | undefined) => import("./response").ResponseTransformer<any>;
        cookie: (name: string, value: string, options?: import("cookie").CookieSerializeOptions | undefined) => import("./response").ResponseTransformer<any>;
        body: <BodyType extends string | Blob | ArrayBufferView | ArrayBuffer | ReadableStream<any> | FormData>(value: BodyType) => import("./response").ResponseTransformer<BodyType>;
        text: <BodyType_1 extends string>(body: BodyType_1) => import("./response").ResponseTransformer<BodyType_1>;
        json: <BodyTypeJSON>(body: BodyTypeJSON, { merge }?: {
            merge?: boolean | undefined;
        }) => import("./response").ResponseTransformer<BodyTypeJSON>;
        xml: <BodyType_2 extends string>(body: BodyType_2) => import("./response").ResponseTransformer<BodyType_2>;
        delay: (durationMs?: number | undefined) => import("./response").ResponseTransformer<any>;
        fetch: (input: string | MockedRequest<DefaultRequestBodyType, RequestParams>, requestInit?: RequestInit) => any;
    }, ParsedRestRequest, MockedRequest<RequestBodyType, RequestParams>, ResponseBodyType>;
};
